package jp.co.sds_corp.tool.check;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Browser
{
	String appName;
	String userAgent;
	String name = "UNKNOWN";
	String version = null;
	Map<String,String> mapProp = new HashMap<String,String>();
	List<String> listProp = new ArrayList<String>();

	Browser(String appName, String userAgent) {
		this.appName = appName;
		this.userAgent = userAgent;
	}

	public static Browser parse(String appName, String userAgent) {
		Browser browser = new Browser(appName, userAgent);
		browser.parse();
		browser.parseBrowser();
		return browser;
	}

	private void parseBrowser() {
		if (appName == null || userAgent == null) {
			return;
		}
		if (appName.equals("Microsoft Internet Explorer")) {
			this.name = appName;
			for (String prop : listProp) {
				if (prop.startsWith("MSIE ")) {
					this.version = prop.substring(5);
					break;
				}
			}
		} else
		if (appName.equals("Opera")) {
			this.name = appName;
			this.version = getProperty("Version");
		} else
		if (getProperty("Firefox") != null) {
			this.name = "Firefox";
			this.version = getProperty("Firefox");
		} else
		if (getProperty("Chrome") != null) {
			this.name = "Google Chrome";
			this.version = getProperty("Chrome");
		} else
		if (getProperty("Safari") != null) {
			this.name = "Safari";
			this.version = getProperty("Version");
		} else
		if (getProperty("Mozilla") != null) {
			this.name = "Mozilla";
			this.version = getProperty("Mozilla");
		} else {
			this.name = appName;
		}
	}

	void parse() {
		parse(userAgent);
	}

	void parse(String str) {
		if (str == null || str.length() == 0) {
			return;
		}
		int start = str.indexOf("(");
		if (start >= 0) {
			parse(str.substring(0, start));
			int end = str.indexOf(")");
			parseList(str.substring(start + 1, end));
			parse(str.substring(end + 1));
			return;
		}

		String[] arr = split(str, " ");
		for (int i = 0; i < arr.length; i++) {
			parseProperty(arr[i].trim());
		}
	}

	void parseProperty(String str) {
		if (str == null || str.length() == 0) {
			return;
		}
		int idx = str.indexOf("/");
		if (idx >= 0) {
			String key = str.substring(0, idx);
			String value = str.substring(idx + 1);
			mapProp.put(key, value);
		} else {
			mapProp.put(str, str);
		}
	}

	void parseList(String str) {
		String[] arr = split(str, ";");
		for (int i = 0; i < arr.length; i++) {
			listProp.add(arr[i].trim());
		}
	}

	public static String[] split(String var, String sep) {
		List<String> list = new ArrayList<String>();
		int pos = 0;
		for (;;) {
			int idx = var.indexOf(sep, pos);
			if (idx < 0) {
				list.add(var.substring(pos));
				break;
			}
			list.add(var.substring(pos, idx));
			pos = idx + sep.length();
		}
		String[] ret = new String[list.size()];
		list.toArray(ret);
		return ret;
	}

	public List<String> listProperties() {
		return listProp;
	}

	public String getProperty(String key) {
		return mapProp.get(key);
	}

	public String getAppName() {
		return appName;
	}

	public String getUserAgent() {
		return userAgent;
	}

	public String getName() {
		return name;
	}
	void setName(String name) {
		this.name = name;
	}

	public String getVersion() {
		return version;
	}
	void setVersion(String version) {
		this.version = version;
	}

	public String toString() {
		String ret = name + " version=";
		if (version != null) {
			ret += version;
		} else {
			ret += "N/A";
		}
		return ret;
	}

}
