package jp.co.sds_corp.tool.check;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Point;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;

public class ProgressDialog extends JDialog implements ProgressListener
{
	private static final long serialVersionUID = 1L;
	int count;
	JLabel label;
	JProgressBar bar;

	public ProgressDialog() {
		createLayout();
	}

	public ProgressDialog(Frame parent) {
		super(parent);
		setModal(false);
		createLayout();
	}

	void createLayout() {
		setModal(false);
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);

		label = new JLabel();
		getContentPane().add(label, BorderLayout.CENTER);
		bar = new JProgressBar();
		getContentPane().add(bar, BorderLayout.SOUTH);
		setPreferredSize(new Dimension(300, 75));
	}

	public void setCheckCount(int count) {
		this.count = count;
		bar.setMaximum(count);
	}

	public void clear() {
		label.setText("");
		bar.setValue(0);
		pack();
	}

	@Override
	public void progressNotify(ProgressListener.Type type, String message) {
		SwingUtilities.invokeLater(new Updator(type, message));
	}

	class Updator implements Runnable {
		ProgressListener.Type type;
		String message;
		Updator(ProgressListener.Type type, String message) {
			this.type = type;
			this.message = message;
		}
		public void run() {
			label.setText(message);
			bar.setString(message);
			if (type == ProgressListener.Type.FORWARD) {
				bar.setValue(bar.getValue() + 1);
			}
		}
	}

	@Override
	public void setVisible(boolean visible) {
		Component c = getParent();
		if (c == null) {
		} else {
			Point windowPos = c.getLocationOnScreen();
			int center_x = c.getWidth() / 2;
			int center_y = c.getHeight() / 2;
			int sx = windowPos.x + center_x - getWidth() / 2;
			int sy = windowPos.y + center_y - getHeight() / 2;
			setLocation(sx, sy);
		}
		super.setVisible(visible);
	}
}
